@@ -571,6 +571,15 @@ pycairo_glyph_path (PycairoContext *o, PyObject *args)
     Py_RETURN_NONE;
 }
 
+static PyObject *
+pycairo_has_current_point (PycairoContext *o)
+{
+    PyObject *b = cairo_has_current_point (o->ctx) ? Py_True : Py_False;
+    RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
+    Py_INCREF(b);
+    return b;
+}
+
 static PyObject *
 pycairo_identity_matrix (PycairoContext *o)
 {
@@ -700,6 +709,15 @@ pycairo_paint_with_alpha (PycairoContext *o, PyObject *args)
     Py_RETURN_NONE;
 }
 
+static PyObject *
+pycairo_path_extents (PycairoContext *o)
+{
+    double x1, y1, x2, y2;
+    cairo_path_extents (o->ctx, &x1, &y1, &x2, &y2);
+    RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
+    return Py_BuildValue("(dddd)", x1, y1, x2, y2);
+}
+
 static PyObject *
 pycairo_pop_group (PycairoContext *o)
 {
@@ -1298,107 +1316,109 @@ static PyMethodDef pycairo_methods[] = {
      * - not needed since Pycairo calls Pycairo_Check_Status() to check
      *   for errors and raise exceptions
      */
-    {"append_path",     (PyCFunction)pycairo_append_path,     METH_VARARGS},
-    {"arc",             (PyCFunction)pycairo_arc,             METH_VARARGS},
-    {"arc_negative",    (PyCFunction)pycairo_arc_negative,    METH_VARARGS},
-    {"clip",            (PyCFunction)pycairo_clip,            METH_NOARGS},
-    {"clip_extents",    (PyCFunction)pycairo_clip_extents,    METH_NOARGS},
-    {"clip_preserve",   (PyCFunction)pycairo_clip_preserve,   METH_NOARGS},
-    {"close_path",      (PyCFunction)pycairo_close_path,      METH_NOARGS},
+    {"append_path",     (PyCFunction)pycairo_append_path,      METH_VARARGS},
+    {"arc",             (PyCFunction)pycairo_arc,              METH_VARARGS},
+    {"arc_negative",    (PyCFunction)pycairo_arc_negative,     METH_VARARGS},
+    {"clip",            (PyCFunction)pycairo_clip,             METH_NOARGS},
+    {"clip_extents",    (PyCFunction)pycairo_clip_extents,     METH_NOARGS},
+    {"clip_preserve",   (PyCFunction)pycairo_clip_preserve,    METH_NOARGS},
+    {"close_path",      (PyCFunction)pycairo_close_path,       METH_NOARGS},
     {"copy_clip_rectangle_list", (PyCFunction)pycairo_copy_clip_rectangle_list,
-                                                              METH_NOARGS},
-    {"copy_page",       (PyCFunction)pycairo_copy_page,       METH_NOARGS},
-    {"copy_path",       (PyCFunction)pycairo_copy_path,       METH_NOARGS},
-    {"copy_path_flat",  (PyCFunction)pycairo_copy_path_flat,  METH_NOARGS},
-    {"curve_to",        (PyCFunction)pycairo_curve_to,        METH_VARARGS},
-    {"device_to_user",  (PyCFunction)pycairo_device_to_user,  METH_VARARGS},
+                                                               METH_NOARGS},
+    {"copy_page",       (PyCFunction)pycairo_copy_page,        METH_NOARGS},
+    {"copy_path",       (PyCFunction)pycairo_copy_path,        METH_NOARGS},
+    {"copy_path_flat",  (PyCFunction)pycairo_copy_path_flat,   METH_NOARGS},
+    {"curve_to",        (PyCFunction)pycairo_curve_to,         METH_VARARGS},
+    {"device_to_user",  (PyCFunction)pycairo_device_to_user,   METH_VARARGS},
     {"device_to_user_distance",
-               (PyCFunction)pycairo_device_to_user_distance,  METH_VARARGS},
-    {"fill",            (PyCFunction)pycairo_fill,            METH_NOARGS},
-    {"fill_extents",    (PyCFunction)pycairo_fill_extents,    METH_NOARGS},
-    {"fill_preserve",   (PyCFunction)pycairo_fill_preserve,   METH_NOARGS},
-    {"font_extents",    (PyCFunction)pycairo_font_extents,    METH_NOARGS},
-    {"get_antialias",   (PyCFunction)pycairo_get_antialias,   METH_NOARGS},
+               (PyCFunction)pycairo_device_to_user_distance,   METH_VARARGS},
+    {"fill",            (PyCFunction)pycairo_fill,             METH_NOARGS},
+    {"fill_extents",    (PyCFunction)pycairo_fill_extents,     METH_NOARGS},
+    {"fill_preserve",   (PyCFunction)pycairo_fill_preserve,    METH_NOARGS},
+    {"font_extents",    (PyCFunction)pycairo_font_extents,     METH_NOARGS},
+    {"get_antialias",   (PyCFunction)pycairo_get_antialias,    METH_NOARGS},
     {"get_current_point",(PyCFunction)pycairo_get_current_point,METH_NOARGS},
-    {"get_dash",        (PyCFunction)pycairo_get_dash,        METH_NOARGS},
-    {"get_dash_count",  (PyCFunction)pycairo_get_dash_count,  METH_NOARGS},
-    {"get_fill_rule",   (PyCFunction)pycairo_get_fill_rule,   METH_NOARGS},
-    {"get_font_face",   (PyCFunction)pycairo_get_font_face,   METH_NOARGS},
-    {"get_font_matrix", (PyCFunction)pycairo_get_font_matrix, METH_NOARGS},
-    {"get_font_options",(PyCFunction)pycairo_get_font_options,METH_NOARGS},
-    {"get_group_target",(PyCFunction)pycairo_get_group_target,METH_NOARGS},
-    {"get_line_cap",    (PyCFunction)pycairo_get_line_cap,    METH_NOARGS},
-    {"get_line_join",   (PyCFunction)pycairo_get_line_join,   METH_NOARGS},
-    {"get_line_width",  (PyCFunction)pycairo_get_line_width,  METH_NOARGS},
-    {"get_matrix",      (PyCFunction)pycairo_get_matrix,      METH_NOARGS},
-    {"get_miter_limit", (PyCFunction)pycairo_get_miter_limit, METH_NOARGS},
-    {"get_operator",    (PyCFunction)pycairo_get_operator,    METH_NOARGS},
-    {"get_scaled_font", (PyCFunction)pycairo_get_scaled_font, METH_NOARGS},
-    {"get_source",      (PyCFunction)pycairo_get_source,      METH_NOARGS},
-    {"get_target",      (PyCFunction)pycairo_get_target,      METH_NOARGS},
-    {"get_tolerance",   (PyCFunction)pycairo_get_tolerance,   METH_NOARGS},
-    {"glyph_extents",   (PyCFunction)pycairo_glyph_extents,   METH_VARARGS},
-    {"glyph_path",      (PyCFunction)pycairo_glyph_path,      METH_VARARGS},
-    {"identity_matrix", (PyCFunction)pycairo_identity_matrix, METH_NOARGS},
-    {"in_fill",         (PyCFunction)pycairo_in_fill,         METH_VARARGS},
-    {"in_stroke",       (PyCFunction)pycairo_in_stroke,       METH_VARARGS},
-    {"line_to",         (PyCFunction)pycairo_line_to,         METH_VARARGS},
-    {"mask",            (PyCFunction)pycairo_mask,            METH_VARARGS},
-    {"mask_surface",    (PyCFunction)pycairo_mask_surface,    METH_VARARGS},
-    {"move_to",         (PyCFunction)pycairo_move_to,         METH_VARARGS},
-    {"new_path",        (PyCFunction)pycairo_new_path,        METH_NOARGS},
-    {"new_sub_path",    (PyCFunction)pycairo_new_sub_path,    METH_NOARGS},
-    {"paint",           (PyCFunction)pycairo_paint,           METH_NOARGS},
-    {"paint_with_alpha",(PyCFunction)pycairo_paint_with_alpha,METH_VARARGS},
-    {"pop_group",       (PyCFunction)pycairo_pop_group,       METH_NOARGS},
+    {"get_dash",        (PyCFunction)pycairo_get_dash,         METH_NOARGS},
+    {"get_dash_count",  (PyCFunction)pycairo_get_dash_count,   METH_NOARGS},
+    {"get_fill_rule",   (PyCFunction)pycairo_get_fill_rule,    METH_NOARGS},
+    {"get_font_face",   (PyCFunction)pycairo_get_font_face,    METH_NOARGS},
+    {"get_font_matrix", (PyCFunction)pycairo_get_font_matrix,  METH_NOARGS},
+    {"get_font_options",(PyCFunction)pycairo_get_font_options, METH_NOARGS},
+    {"get_group_target",(PyCFunction)pycairo_get_group_target, METH_NOARGS},
+    {"get_line_cap",    (PyCFunction)pycairo_get_line_cap,     METH_NOARGS},
+    {"get_line_join",   (PyCFunction)pycairo_get_line_join,    METH_NOARGS},
+    {"get_line_width",  (PyCFunction)pycairo_get_line_width,   METH_NOARGS},
+    {"get_matrix",      (PyCFunction)pycairo_get_matrix,       METH_NOARGS},
+    {"get_miter_limit", (PyCFunction)pycairo_get_miter_limit,  METH_NOARGS},
+    {"get_operator",    (PyCFunction)pycairo_get_operator,     METH_NOARGS},
+    {"get_scaled_font", (PyCFunction)pycairo_get_scaled_font,  METH_NOARGS},
+    {"get_source",      (PyCFunction)pycairo_get_source,       METH_NOARGS},
+    {"get_target",      (PyCFunction)pycairo_get_target,       METH_NOARGS},
+    {"get_tolerance",   (PyCFunction)pycairo_get_tolerance,    METH_NOARGS},
+    {"glyph_extents",   (PyCFunction)pycairo_glyph_extents,    METH_VARARGS},
+    {"glyph_path",      (PyCFunction)pycairo_glyph_path,       METH_VARARGS},
+    {"has_current_point",(PyCFunction)pycairo_has_current_point, METH_NOARGS},
+    {"identity_matrix", (PyCFunction)pycairo_identity_matrix,  METH_NOARGS},
+    {"in_fill",         (PyCFunction)pycairo_in_fill,          METH_VARARGS},
+    {"in_stroke",       (PyCFunction)pycairo_in_stroke,        METH_VARARGS},
+    {"line_to",         (PyCFunction)pycairo_line_to,          METH_VARARGS},
+    {"mask",            (PyCFunction)pycairo_mask,             METH_VARARGS},
+    {"mask_surface",    (PyCFunction)pycairo_mask_surface,     METH_VARARGS},
+    {"move_to",         (PyCFunction)pycairo_move_to,          METH_VARARGS},
+    {"new_path",        (PyCFunction)pycairo_new_path,         METH_NOARGS},
+    {"new_sub_path",    (PyCFunction)pycairo_new_sub_path,     METH_NOARGS},
+    {"paint",           (PyCFunction)pycairo_paint,            METH_NOARGS},
+    {"paint_with_alpha",(PyCFunction)pycairo_paint_with_alpha, METH_VARARGS},
+    {"path_extents",    (PyCFunction)pycairo_path_extents,     METH_NOARGS},
+    {"pop_group",       (PyCFunction)pycairo_pop_group,        METH_NOARGS},
     {"pop_group_to_source",
-                    (PyCFunction)pycairo_pop_group_to_source, METH_NOARGS},
-    {"push_group",      (PyCFunction)pycairo_push_group,      METH_NOARGS},
+                    (PyCFunction)pycairo_pop_group_to_source,  METH_NOARGS},
+    {"push_group",      (PyCFunction)pycairo_push_group,       METH_NOARGS},
     {"push_group_with_content",
-                (PyCFunction)pycairo_push_group_with_content, METH_VARARGS},
-    {"rectangle",       (PyCFunction)pycairo_rectangle,       METH_VARARGS},
-    {"rel_curve_to",    (PyCFunction)pycairo_rel_curve_to,    METH_VARARGS},
-    {"rel_line_to",     (PyCFunction)pycairo_rel_line_to,     METH_VARARGS},
-    {"rel_move_to",     (PyCFunction)pycairo_rel_move_to,     METH_VARARGS},
-    {"reset_clip",      (PyCFunction)pycairo_reset_clip,      METH_NOARGS},
-    {"restore",         (PyCFunction)pycairo_restore,         METH_NOARGS},
-    {"rotate",          (PyCFunction)pycairo_rotate,          METH_VARARGS},
-    {"save",            (PyCFunction)pycairo_save,            METH_NOARGS},
-    {"scale",           (PyCFunction)pycairo_scale,           METH_VARARGS},
-    {"select_font_face",(PyCFunction)pycairo_select_font_face,METH_VARARGS},
-    {"set_antialias",   (PyCFunction)pycairo_set_antialias,   METH_VARARGS},
-    {"set_dash",        (PyCFunction)pycairo_set_dash,        METH_VARARGS},
-    {"set_fill_rule",   (PyCFunction)pycairo_set_fill_rule,   METH_VARARGS},
-    {"set_font_face",   (PyCFunction)pycairo_set_font_face,   METH_O},
-    {"set_font_matrix", (PyCFunction)pycairo_set_font_matrix, METH_VARARGS},
-    {"set_font_options",(PyCFunction)pycairo_set_font_options,METH_VARARGS},
-    {"set_font_size",   (PyCFunction)pycairo_set_font_size,   METH_VARARGS},
-    {"set_line_cap",    (PyCFunction)pycairo_set_line_cap,    METH_VARARGS},
-    {"set_line_join",   (PyCFunction)pycairo_set_line_join,   METH_VARARGS},
-    {"set_line_width",  (PyCFunction)pycairo_set_line_width,  METH_VARARGS},
-    {"set_matrix",      (PyCFunction)pycairo_set_matrix,      METH_VARARGS},
-    {"set_miter_limit", (PyCFunction)pycairo_set_miter_limit, METH_VARARGS},
-    {"set_operator",    (PyCFunction)pycairo_set_operator,    METH_VARARGS},
-    /* set_scaled_font */
-    {"set_source",      (PyCFunction)pycairo_set_source,      METH_VARARGS},
-    {"set_source_rgb",  (PyCFunction)pycairo_set_source_rgb,  METH_VARARGS},
-    {"set_source_rgba", (PyCFunction)pycairo_set_source_rgba, METH_VARARGS},
+                (PyCFunction)pycairo_push_group_with_content,  METH_VARARGS},
+    {"rectangle",       (PyCFunction)pycairo_rectangle,        METH_VARARGS},
+    {"rel_curve_to",    (PyCFunction)pycairo_rel_curve_to,     METH_VARARGS},
+    {"rel_line_to",     (PyCFunction)pycairo_rel_line_to,      METH_VARARGS},
+    {"rel_move_to",     (PyCFunction)pycairo_rel_move_to,      METH_VARARGS},
+    {"reset_clip",      (PyCFunction)pycairo_reset_clip,       METH_NOARGS},
+    {"restore",         (PyCFunction)pycairo_restore,          METH_NOARGS},
+    {"rotate",          (PyCFunction)pycairo_rotate,           METH_VARARGS},
+    {"save",            (PyCFunction)pycairo_save,             METH_NOARGS},
+    {"scale",           (PyCFunction)pycairo_scale,            METH_VARARGS},
+    {"select_font_face",(PyCFunction)pycairo_select_font_face, METH_VARARGS},
+    {"set_antialias",   (PyCFunction)pycairo_set_antialias,    METH_VARARGS},
+    {"set_dash",        (PyCFunction)pycairo_set_dash,         METH_VARARGS},
+    {"set_fill_rule",   (PyCFunction)pycairo_set_fill_rule,    METH_VARARGS},
+    {"set_font_face",   (PyCFunction)pycairo_set_font_face,    METH_O},
+    {"set_font_matrix", (PyCFunction)pycairo_set_font_matrix,  METH_VARARGS},
+    {"set_font_options",(PyCFunction)pycairo_set_font_options, METH_VARARGS},
+    {"set_font_size",   (PyCFunction)pycairo_set_font_size,    METH_VARARGS},
+    {"set_line_cap",    (PyCFunction)pycairo_set_line_cap,     METH_VARARGS},
+    {"set_line_join",   (PyCFunction)pycairo_set_line_join,    METH_VARARGS},
+    {"set_line_width",  (PyCFunction)pycairo_set_line_width,   METH_VARARGS},
+    {"set_matrix",      (PyCFunction)pycairo_set_matrix,       METH_VARARGS},
+    {"set_miter_limit", (PyCFunction)pycairo_set_miter_limit,  METH_VARARGS},
+    {"set_operator",    (PyCFunction)pycairo_set_operator,     METH_VARARGS},
+    /* set_scaled_font - not implemented yet */
+    {"set_source",      (PyCFunction)pycairo_set_source,       METH_VARARGS},
+    {"set_source_rgb",  (PyCFunction)pycairo_set_source_rgb,   METH_VARARGS},
+    {"set_source_rgba", (PyCFunction)pycairo_set_source_rgba,  METH_VARARGS},
     {"set_source_surface",(PyCFunction)pycairo_set_source_surface,
-                                                              METH_VARARGS},
-    {"set_tolerance",   (PyCFunction)pycairo_set_tolerance,   METH_VARARGS},
-    {"show_glyphs",     (PyCFunction)pycairo_show_glyphs,     METH_VARARGS},
-    {"show_page",       (PyCFunction)pycairo_show_page,       METH_NOARGS},
-    {"show_text",       (PyCFunction)pycairo_show_text,       METH_O},
-    {"stroke",          (PyCFunction)pycairo_stroke,          METH_NOARGS},
-    {"stroke_extents",  (PyCFunction)pycairo_stroke_extents,  METH_NOARGS},
-    {"stroke_preserve", (PyCFunction)pycairo_stroke_preserve, METH_NOARGS},
-    {"text_extents",    (PyCFunction)pycairo_text_extents,    METH_O},
-    {"text_path",       (PyCFunction)pycairo_text_path,       METH_O},
-    {"transform",       (PyCFunction)pycairo_transform,       METH_VARARGS},
-    {"translate",       (PyCFunction)pycairo_translate,       METH_VARARGS},
-    {"user_to_device",  (PyCFunction)pycairo_user_to_device,  METH_VARARGS},
+                                                               METH_VARARGS},
+    {"set_tolerance",   (PyCFunction)pycairo_set_tolerance,    METH_VARARGS},
+    {"show_glyphs",     (PyCFunction)pycairo_show_glyphs,      METH_VARARGS},
+    {"show_page",       (PyCFunction)pycairo_show_page,        METH_NOARGS},
+    {"show_text",       (PyCFunction)pycairo_show_text,        METH_O},
+    {"stroke",          (PyCFunction)pycairo_stroke,           METH_NOARGS},
+    {"stroke_extents",  (PyCFunction)pycairo_stroke_extents,   METH_NOARGS},
+    {"stroke_preserve", (PyCFunction)pycairo_stroke_preserve,  METH_NOARGS},
+    {"text_extents",    (PyCFunction)pycairo_text_extents,     METH_O},
+    {"text_path",       (PyCFunction)pycairo_text_path,        METH_O},
+    {"transform",       (PyCFunction)pycairo_transform,        METH_VARARGS},
+    {"translate",       (PyCFunction)pycairo_translate,        METH_VARARGS},
+    {"user_to_device",  (PyCFunction)pycairo_user_to_device,   METH_VARARGS},
     {"user_to_device_distance",(PyCFunction)pycairo_user_to_device_distance,
-                                                              METH_VARARGS},
+                                                               METH_VARARGS},
     {NULL, NULL, 0, NULL},
 };
 
